@@ -46,27 +46,22 @@
 #include "ares_dns.h"
 
 #ifndef HAVE_STRDUP
-#  include "ares_str.h"
+#  include "str/ares_str.h"
 #  define strdup(ptr) ares_strdup(ptr)
 #endif
 
 #ifndef HAVE_STRCASECMP
-#  include "ares_strcasecmp.h"
+#  include "str/ares_strcasecmp.h"
 #  define strcasecmp(p1, p2) ares_strcasecmp(p1, p2)
 #endif
 
 #ifndef HAVE_STRNCASECMP
-#  include "ares_strcasecmp.h"
+#  include "str/ares_strcasecmp.h"
 #  define strncasecmp(p1, p2, n) ares_strncasecmp(p1, p2, n)
 #endif
 
 #include "ares_getopt.h"
 
-#ifdef WATT32
-#  undef WIN32 /* Redefined in MingW headers */
-#endif
-
-
 typedef struct {
   ares_bool_t         is_help;
   struct ares_options options;
@@ -84,12 +79,14 @@ typedef struct {
 } nv_t;
 
 static const nv_t configflags[] = {
-  {"usevc",      ARES_FLAG_USEVC    },
-  { "primary",   ARES_FLAG_PRIMARY  },
-  { "igntc",     ARES_FLAG_IGNTC    },
-  { "norecurse", ARES_FLAG_NORECURSE},
-  { "stayopen",  ARES_FLAG_STAYOPEN },
-  { "noaliases", ARES_FLAG_NOALIASES}
+  { "usevc",     ARES_FLAG_USEVC     },
+  { "primary",   ARES_FLAG_PRIMARY   },
+  { "igntc",     ARES_FLAG_IGNTC     },
+  { "norecurse", ARES_FLAG_NORECURSE },
+  { "stayopen",  ARES_FLAG_STAYOPEN  },
+  { "noaliases", ARES_FLAG_NOALIASES },
+  { "edns",      ARES_FLAG_EDNS      },
+  { "dns0x20",   ARES_FLAG_DNS0x20   }
 };
 static const size_t nconfigflags = sizeof(configflags) / sizeof(*configflags);
 
@@ -118,35 +115,43 @@ static void free_config(adig_config_t *config)
 
 static void print_help(void)
 {
+  /* Split due to maximum c89 string literal of 509 bytes */
   printf("adig version %s\n\n", ares_version(NULL));
   printf(
     "usage: adig [-h] [-d] [-f flag] [[-s server] ...] [-T|U port] [-c class]\n"
-    "            [-t type] name ...\n\n"
-    "  -h : Display this help and exit.\n"
-    "  -d : Print some extra debugging output.\n"
-    "  -f flag   : Add a behavior control flag. Possible values are\n"
+    "            [-t type] name ...\n\n");
+  printf("  -h : Display this help and exit.\n");
+  printf("  -d : Print some extra debugging output.\n");
+  printf(
+    "  -f flag   : Add a behavior control flag. May be specified more than "
+    "once\n"
+    "              to add additional flags. Possible values are:\n"
     "              igntc     - do not retry a truncated query as TCP, just\n"
     "                          return the truncated answer\n"
     "              noaliases - don't honor the HOSTALIASES environment\n"
-    "                          variable\n"
-    "              norecurse - don't query upstream servers recursively\n"
-    "              primary   - use the first server\n"
-    "              stayopen  - don't close the communication sockets\n"
-    "              usevc     - use TCP only\n"
+    "                          variable\n");
+  printf("              norecurse - don't query upstream servers recursively\n"
+         "              primary   - use the first server\n"
+         "              stayopen  - don't close the communication sockets\n"
+         "              usevc     - use TCP only\n"
+         "              edns      - use EDNS\n"
+         "              dns0x20   - enable DNS 0x20 support\n");
+  printf(
     "  -s server : Connect to the specified DNS server, instead of the\n"
     "              system's default one(s). Servers are tried in round-robin,\n"
-    "              if the previous one failed.\n"
-    "  -T port   : Connect to the specified TCP port of DNS server.\n"
-    "  -U port   : Connect to the specified UDP port of DNS server.\n"
-    "  -c class  : Set the query class. Possible values for class are:\n"
-    "              ANY, CHAOS, HS and IN (default)\n"
+    "              if the previous one failed.\n");
+  printf("  -T port   : Connect to the specified TCP port of DNS server.\n");
+  printf("  -U port   : Connect to the specified UDP port of DNS server.\n");
+  printf("  -c class  : Set the query class. Possible values for class are:\n"
+         "              ANY, CHAOS, HS and IN (default)\n");
+  printf(
     "  -t type   : Query records of the specified type. Possible values for\n"
     "              type are:\n"
     "              A (default), AAAA, ANY, CNAME, HINFO, MX, NAPTR, NS, PTR,\n"
     "              SOA, SRV, TXT, TLSA, URI, CAA, SVCB, HTTPS\n\n");
 }
 
-static ares_bool_t read_cmdline(int argc, const char **argv,
+static ares_bool_t read_cmdline(int argc, const char * const *argv,
                                 adig_config_t *config)
 {
   ares_getopt_state_t state;
@@ -209,26 +214,35 @@ static ares_bool_t read_cmdline(int argc, const char **argv,
         break;
 
       case 'T':
-        /* Set the TCP port number. */
-        if (!isdigit(*state.optarg)) {
-          snprintf(config->error, sizeof(config->error), "invalid port number");
-          return ARES_FALSE;
+        {
+          /* Set the TCP port number. */
+          long port = strtol(state.optarg, NULL, 0);
+
+          if (port <= 0 || port > 65535) {
+            snprintf(config->error, sizeof(config->error),
+                     "invalid port number");
+            return ARES_FALSE;
+          }
+          config->options.tcp_port  = (unsigned short)port;
+          config->options.flags    |= ARES_FLAG_USEVC;
+          config->optmask          |= ARES_OPT_TCP_PORT;
         }
-        config->options.tcp_port =
-          (unsigned short)strtol(state.optarg, NULL, 0);
-        config->options.flags |= ARES_FLAG_USEVC;
-        config->optmask       |= ARES_OPT_TCP_PORT;
         break;
 
       case 'U':
-        /* Set the UDP port number. */
-        if (!isdigit(*state.optarg)) {
-          snprintf(config->error, sizeof(config->error), "invalid port number");
-          return ARES_FALSE;
+        {
+          /* Set the TCP port number. */
+          long port = strtol(state.optarg, NULL, 0);
+
+          if (port <= 0 || port > 65535) {
+            snprintf(config->error, sizeof(config->error),
+                     "invalid port number");
+            return ARES_FALSE;
+          }
+          config->options.udp_port  = (unsigned short)port;
+          config->options.flags    |= ARES_FLAG_USEVC;
+          config->optmask          |= ARES_OPT_UDP_PORT;
         }
-        config->options.udp_port =
-          (unsigned short)strtol(state.optarg, NULL, 0);
-        config->optmask |= ARES_OPT_UDP_PORT;
         break;
 
       case ':':
@@ -618,6 +632,21 @@ static void print_binp(const ares_dns_rr_t *rr, ares_dns_rr_key_t key)
   print_opt_binp(binp, len);
 }
 
+static void print_abinp(const ares_dns_rr_t *rr, ares_dns_rr_key_t key)
+{
+  size_t i;
+  size_t cnt = ares_dns_rr_get_abin_cnt(rr, key);
+
+  for (i = 0; i < cnt; i++) {
+    size_t               len;
+    const unsigned char *binp = ares_dns_rr_get_abin(rr, key, i, &len);
+    if (i != 0) {
+      printf(" ");
+    }
+    print_opt_binp(binp, len);
+  }
+}
+
 static void print_rr(const ares_dns_rr_t *rr)
 {
   const char              *name     = ares_dns_rr_get_name(rr);
@@ -677,6 +706,9 @@ static void print_rr(const ares_dns_rr_t *rr)
       case ARES_DATATYPE_BINP:
         print_binp(rr, keys[i]);
         break;
+      case ARES_DATATYPE_ABINP:
+        print_abinp(rr, keys[i]);
+        break;
       case ARES_DATATYPE_OPT:
         print_opts(rr, keys[i]);
         break;
@@ -722,18 +754,31 @@ static void print_section(ares_dns_record_t *dnsrec, ares_dns_section_t section)
 
 static void print_opt_psuedosection(ares_dns_record_t *dnsrec)
 {
-  const ares_dns_rr_t *rr = has_opt(dnsrec, ARES_SECTION_ADDITIONAL);
+  const ares_dns_rr_t *rr         = has_opt(dnsrec, ARES_SECTION_ADDITIONAL);
+  const unsigned char *cookie     = NULL;
+  size_t               cookie_len = 0;
+
   if (rr == NULL) {
     return;
   }
 
+  if (!ares_dns_rr_get_opt_byid(rr, ARES_RR_OPT_OPTIONS, ARES_OPT_PARAM_COOKIE,
+                                &cookie, &cookie_len)) {
+    cookie = NULL;
+  }
+
+
   printf(";; OPT PSEUDOSECTION:\n");
-  printf("; EDNS: version: %u, flags: %u; udp: %u\t",
+  printf("; EDNS: version: %u, flags: %u; udp: %u\n",
          (unsigned int)ares_dns_rr_get_u8(rr, ARES_RR_OPT_VERSION),
          (unsigned int)ares_dns_rr_get_u16(rr, ARES_RR_OPT_FLAGS),
          (unsigned int)ares_dns_rr_get_u16(rr, ARES_RR_OPT_UDP_SIZE));
 
-  printf("\n");
+  if (cookie) {
+    printf("; COOKIE: ");
+    print_opt_bin(cookie, cookie_len);
+    printf(" (good)\n");
+  }
 }
 
 static void callback(void *arg, int status, int timeouts, unsigned char *abuf,
@@ -745,9 +790,9 @@ static void callback(void *arg, int status, int timeouts, unsigned char *abuf,
 
   /* We got a "Server status" */
   if (status >= ARES_SUCCESS && status <= ARES_EREFUSED) {
-     printf(";; Got answer:");
+    printf(";; Got answer:");
   } else {
-     printf(";;");
+    printf(";;");
   }
 
   if (status != ARES_SUCCESS) {
@@ -902,7 +947,7 @@ int main(int argc, char **argv)
   memset(&config, 0, sizeof(config));
   config.qclass = ARES_CLASS_IN;
   config.qtype  = ARES_REC_TYPE_A;
-  if (!read_cmdline(argc, (const char **)argv, &config)) {
+  if (!read_cmdline(argc, (const char * const *)argv, &config)) {
     printf("\n** ERROR: %s\n\n", config.error);
     print_help();
     rv = 1;
