@@ -19,7 +19,8 @@ unset CPPFLAGS
 # because it's not available on earlier releases and leads to
 # segfaults because the symbol clock_gettime is NULL.
 # See https://github.com/gevent/gevent/issues/916
-export CPPFLAGS="-D_DARWIN_FEATURE_CLOCK_GETTIME=0"
+#export CPPFLAGS="-D_DARWIN_FEATURE_CLOCK_GETTIME=0"
+export ARCHFLAGS="-arch arm64 -arch x86_64"
 
 BASE=`pwd`/../../
 BASE=`greadlink -f $BASE`
@@ -35,10 +36,5 @@ echo cloning $BASE
 git clone $BASE gevent
 cd ./gevent
 pip install -U pip
-pip install -U setuptools greenlet cffi
-pip install -U wheel
-# We may need different versions of deps depending on the
-# version of python; that's captured in this file.
-pip install -U -r dev-requirements.txt
-python ./setup.py sdist bdist_wheel
-cp dist/*whl /tmp/gevent/
+pip wheel . -w dist
+cp dist/gevent*whl /tmp/gevent/
