@@ -241,6 +241,8 @@ class _HostsResolver(object):
         rrset = dns.rrset.RRset(qname, rdclass, rdtype)
         rrset.ttl = self._last_load + self.interval - now
 
+        mapping = None
+        kind = None
         if rdtype == dns.rdatatype.A:
             mapping = hosts_file.v4
             kind = dns.rdtypes.IN.A.A
@@ -270,7 +272,7 @@ class _HostsResolver(object):
         # gethostbyname_ex so it's probably fine.
         aliases = self.hosts_file.aliases
         result = []
-        if hostname in aliases:
+        if hostname in aliases: # pylint:disable=consider-using-get
             cannon = aliases[hostname]
         else:
             cannon = hostname
